@@ -1,12 +1,12 @@
 #
-# Revelation 0.4.5 - a password manager for GNOME 2
+# Revelation 0.4.7 - a password manager for GNOME 2
 # http://oss.codepoet.no/revelation/
-# $Id: __init__.py 431 2005-08-24 13:50:15Z erikg $
+# $Id: __init__.py 478 2006-02-03 14:04:57Z erikg $
 #
 # Module for data handlers
 #
 #
-# Copyright (c) 2003-2005 Erik Grinaker
+# Copyright (c) 2003-2006 Erik Grinaker
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -23,7 +23,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #
 
-from base import HandlerError, DataError, FormatError, PasswordError, VersionError
+from base import Error, DataError, FormatError, PasswordError, VersionError
 
 from fpm import FPM
 from gpass import GPass04, GPass05
@@ -50,7 +50,7 @@ HANDLERS = [
 ]
 
 
-class DetectError(Exception):
+class DetectError(Error):
 	"Exception for autodetection error"
 	pass
 
