@@ -1,12 +1,12 @@
 #
-# Revelation 0.4.5 - a password manager for GNOME 2
+# Revelation 0.4.7 - a password manager for GNOME 2
 # http://oss.codepoet.no/revelation/
-# $Id: dialog.py 419 2005-08-22 22:36:39Z erikg $
+# $Id: dialog.py 476 2006-02-02 19:28:56Z erikg $
 #
 # Module with dialogs
 #
 #
-# Copyright (c) 2003-2005 Erik Grinaker
+# Copyright (c) 2003-2006 Erik Grinaker
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -29,9 +29,6 @@ import gobject, gtk, gnome.ui, urllib
 
 
 
-RESPONSE_NEXT		= 10
-RESPONSE_PREVIOUS	= 11
-
 EVENT_FILTER		= None
 UNIQUE_DIALOGS		= {}
 
@@ -475,6 +472,10 @@ class ExportFileSelector(FileSelector):
 		for handler in datahandler.get_export_handlers():
 			self.dropdown.append_item(handler.name, None, handler)
 
+		for index in range(self.dropdown.get_num_items()):
+			if self.dropdown.get_item(index)[2] == datahandler.RevelationXML:
+				self.dropdown.set_active(index)
+
 
 	def run(self):
 		"Displays the dialog"
@@ -612,30 +613,9 @@ class PasswordChange(Password):
 		if password is not None:
 			self.entry_current = self.add_entry("Current password")
 
-		self.entry_new = self.add_entry("New password", ui.PasswordEntry())
-		self.entry_confirm = self.add_entry("Confirm password")
-
-		self.entry_confirm.connect("changed", self.__cb_check_match)
-		self.entry_confirm.connect("focus-in-event", self.__cb_check_match)
-		self.entry_confirm.connect("focus-out-event", self.__cb_check_match)
-
-
-	def __cb_check_match(self, widget, data = None):
-		"Checks if passwords match"
-
-		password = self.entry_new.get_text()
-		confirm = self.entry_confirm.get_text()
-
-		if len(confirm) == 0 or self.entry_confirm.is_focus() == False:
-			color = ui.Entry().rc_get_style().base[gtk.STATE_NORMAL]
-
-		elif password == confirm:
-			color = gtk.gdk.color_parse("#baffba")
-
-		else:
-			color = gtk.gdk.color_parse("#ffbaba")
-
-		self.entry_confirm.modify_base(gtk.STATE_NORMAL, color)
+		self.entry_new		= self.add_entry("New password", ui.PasswordEntry())
+		self.entry_confirm	= self.add_entry("Confirm password")
+		self.entry_confirm.autocheck = False
 
 
 	def run(self):
@@ -760,28 +740,7 @@ class PasswordSave(Password):
 
 		self.entry_new		= self.add_entry("New password", ui.PasswordEntry())
 		self.entry_confirm	= self.add_entry("Confirm password")
-
-		self.entry_confirm.connect("changed", self.__cb_check_match)
-		self.entry_confirm.connect("focus-in-event", self.__cb_check_match)
-		self.entry_confirm.connect("focus-out-event", self.__cb_check_match)
-
-
-	def __cb_check_match(self, widget, data = None):
-		"Checks if passwords match"
-
-		password = self.entry_new.get_text()
-		confirm = self.entry_confirm.get_text()
-
-		if len(confirm) == 0 or self.entry_confirm.is_focus() == False:
-			color = ui.Entry().rc_get_style().base[gtk.STATE_NORMAL]
-
-		elif password == confirm:
-			color = gtk.gdk.color_parse("#baffba")
-
-		else:
-			color = gtk.gdk.color_parse("#ffbaba")
-
-		self.entry_confirm.modify_base(gtk.STATE_NORMAL, color)
+		self.entry_confirm.autocheck = False
 
 
 	def run(self):
@@ -829,7 +788,7 @@ class EntryEdit(Utility):
 			self, parent, title,
 			(
 				( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL ),
-				( e is None and ui.STOCK_NEW_ENTRY or ui.STOCK_EDIT, gtk.RESPONSE_OK )
+				( e is None and ui.STOCK_NEW_ENTRY or ui.STOCK_UPDATE, gtk.RESPONSE_OK )
 			)
 		)
 
@@ -890,14 +849,13 @@ class EntryEdit(Utility):
 			if self.fielddata.has_key(type(field)):
 				fieldentry.set_text(self.fielddata[type(field)])
 
-			if (fieldentry.flags() & gtk.NO_WINDOW) == gtk.NO_WINDOW:
-				eventbox = ui.EventBox(fieldentry)
-				self.tooltips.set_tip(eventbox, field.description)
-				self.sect_fields.append_widget(field.name, eventbox)
-
-			else:
+			if (fieldentry.flags() & gtk.NO_WINDOW) != gtk.NO_WINDOW:
 				self.tooltips.set_tip(fieldentry, field.description)
-				self.sect_fields.append_widget(field.name, fieldentry)
+
+			elif hasattr(fieldentry, "entry") == True:
+				self.tooltips.set_tip(fieldentry.entry, field.description)
+
+			self.sect_fields.append_widget(field.name, fieldentry)
 
 
 		# show widgets
@@ -1006,7 +964,7 @@ class FolderEdit(Utility):
 			self, parent, title,
 			(
 				( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL ),
-				( e == None and ui.STOCK_NEW_FOLDER or ui.STOCK_EDIT, gtk.RESPONSE_OK )
+				( e == None and ui.STOCK_NEW_FOLDER or ui.STOCK_UPDATE, gtk.RESPONSE_OK )
 			)
 		)
 
@@ -1116,78 +1074,6 @@ class Exception(Error):
 
 
 
-class Find(Utility):
-	"A find dialog"
-
-	def __init__(self, parent, cfg):
-		Utility.__init__(
-			self, parent, "Find an entry",
-			( ( gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE ), ( ui.STOCK_PREVIOUS, RESPONSE_PREVIOUS ), ( ui.STOCK_NEXT, RESPONSE_NEXT ) )
-		)
-
-		self.config = cfg
-		self.set_modal(False)
-
-		section = self.add_section("Find an entry")
-
-		# search string entry
-		self.entry_string = ui.Entry()
-		self.entry_string.connect("changed", lambda w: self.__state_buttons(self.entry_string.get_text() != ""))
-
-		section.append_widget("Search for", self.entry_string)
-		self.tooltips.set_tip(self.entry_string, "The text to search for")
-
-		# account type dropdown
-		self.dropdown = ui.EntryDropDown()
-		self.dropdown.insert_item(0, "Any", "gnome-stock-about")
-
-		eventbox = ui.EventBox(self.dropdown)
-		section.append_widget("Account type", eventbox)
-		self.tooltips.set_tip(eventbox, "The account type to search for")
-
-		# folders checkbutton
-		self.check_folders = ui.CheckButton("Search for folders as well")
-		ui.config_bind(self.config, "search/folders", self.check_folders)
-
-		section.append_widget(None, self.check_folders)
-		self.tooltips.set_tip(self.check_folders, "When enabled, folder names and descriptions will also be searched")
-
-		# namedesc checkbutton
-		self.check_namedesc = ui.CheckButton("Only search in name and description")
-		ui.config_bind(self.config, "search/namedesc", self.check_namedesc)
-
-		section.append_widget(None, self.check_namedesc)
-		self.tooltips.set_tip(self.check_namedesc, "When enabled, only entry names and descriptions will be searched")
-
-		# case sensitive checkbutton
-		self.check_casesensitive = ui.CheckButton("Case sensitive")
-		ui.config_bind(self.config, "search/casesens", self.check_casesensitive)
-
-		section.append_widget(None, self.check_casesensitive)
-		self.tooltips.set_tip(self.check_casesensitive, "When enabled, searches will be case sensitive")
-
-		# set initial state
-		self.__state_buttons(False)
-
-
-	def __state_buttons(self, active):
-		"Sets button sensitivity based on entry contents"
-
-		self.get_button(0).set_sensitive(active)
-		self.get_button(1).set_sensitive(active)
-
-
-	def run(self):
-		"Displays the dialog"
-
-		self.show_all()
-		self.entry_string.grab_focus()
-
-		if EVENT_FILTER != None:
-			self.window.add_filter(EVENT_FILTER)
-
-
-
 class PasswordChecker(Utility):
 	"A password checker dialog"
 
@@ -1229,18 +1115,11 @@ class PasswordChecker(Utility):
 			else:
 				util.check_password(password)
 				icon	= ui.STOCK_PASSWORD_STRONG
-				result	= "The password is good"
+				result	= "The password seems good"
 
 		except ValueError, result:
-			result	= str(result)
-			result = result.replace("simplistic/systematic", "systematic")
-			result = result.replace(" dictionary", "")
-
-			if result[:3] == "it ":
-				result = result[3:]
-
 			icon	= ui.STOCK_PASSWORD_WEAK
-			result = "The password " + result
+			result = "The password " + str(result)
 
 		self.result.set_text(result)
 		self.result.set_stock(icon, ui.ICON_SIZE_LABEL)
