@@ -1,12 +1,12 @@
 #
-# Revelation 0.4.5 - a password manager for GNOME 2
+# Revelation 0.4.7 - a password manager for GNOME 2
 # http://oss.codepoet.no/revelation/
-# $Id: ui.py 412 2005-08-09 10:05:16Z erikg $
+# $Id: ui.py 476 2006-02-02 19:28:56Z erikg $
 #
 # Module for UI functionality
 #
 #
-# Copyright (c) 2003-2005 Erik Grinaker
+# Copyright (c) 2003-2006 Erik Grinaker
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -51,6 +51,7 @@ STOCK_RELOAD			= "revelation-reload"
 STOCK_REMOVE			= "revelation-remove"
 STOCK_UNKNOWN			= "revelation-unknown"
 STOCK_UNLOCK			= "revelation-unlock"
+STOCK_UPDATE			= "revelation-update"
 
 
 STOCK_ENTRY_FOLDER		= "revelation-account-folder"
@@ -70,30 +71,13 @@ STOCK_REVELATION		= "revelation-revelation"
 STOCK_REVELATION_LOCKED		= "revelation-revelation-locked"
 
 
-ICON_SIZE_APPLET		= gtk.icon_size_from_name("revelation-applet")
-ICON_SIZE_DATAVIEW		= gtk.icon_size_from_name("revelation-dataview")
-ICON_SIZE_DROPDOWN		= gtk.icon_size_from_name("revelation-dropdown")
-ICON_SIZE_LABEL			= gtk.icon_size_from_name("revelation-label")
-ICON_SIZE_LOGO			= gtk.icon_size_from_name("revelation-logo")
-ICON_SIZE_TREEVIEW		= gtk.icon_size_from_name("revelation-treeview")
-
-if ICON_SIZE_APPLET == gtk.ICON_SIZE_INVALID:
-	ICON_SIZE_APPLET	= gtk.icon_size_register("revelation-applet", 24, 24)
-
-if ICON_SIZE_DATAVIEW == gtk.ICON_SIZE_INVALID:
-	ICON_SIZE_DATAVIEW	= gtk.icon_size_register("revelation-dataview", 24, 24)
-
-if ICON_SIZE_DROPDOWN == gtk.ICON_SIZE_INVALID:
-	ICON_SIZE_DROPDOWN	= gtk.icon_size_register("revelation-dropdown", 18, 18)
-
-if ICON_SIZE_LABEL == gtk.ICON_SIZE_INVALID:
-	ICON_SIZE_LABEL		= gtk.icon_size_register("revelation-label", 24, 24)
-
-if ICON_SIZE_LOGO == gtk.ICON_SIZE_INVALID:
-	ICON_SIZE_LOGO		= gtk.icon_size_register("revelation-logo", 32, 32)
-
-if ICON_SIZE_TREEVIEW == gtk.ICON_SIZE_INVALID:
-	ICON_SIZE_TREEVIEW	= gtk.icon_size_register("revelation-treeview", 18, 18)
+ICON_SIZE_APPLET		= gtk.ICON_SIZE_LARGE_TOOLBAR
+ICON_SIZE_DATAVIEW		= gtk.ICON_SIZE_LARGE_TOOLBAR
+ICON_SIZE_DROPDOWN		= gtk.ICON_SIZE_SMALL_TOOLBAR
+ICON_SIZE_ENTRY			= gtk.ICON_SIZE_MENU
+ICON_SIZE_LABEL			= gtk.ICON_SIZE_LARGE_TOOLBAR
+ICON_SIZE_LOGO			= gtk.ICON_SIZE_DND
+ICON_SIZE_TREEVIEW		= gtk.ICON_SIZE_SMALL_TOOLBAR
 
 
 
@@ -192,6 +176,14 @@ def generate_field_edit_widget(field, cfg = None, userdata = None):
 
 ##### CONTAINERS #####
 
+class Alignment(gtk.Alignment):
+	"A container bin"
+
+	def __init__(self, xalign = 0, yalign = 0, xscale = 0, yscale = 0):
+		gtk.Alignment.__init__(self, xalign, yalign, xscale, yscale)
+
+
+
 class HBox(gtk.HBox):
 	"A horizontal container"
 
@@ -306,13 +298,27 @@ class Toolbar(gtk.Toolbar):
 	def __init__(self):
 		gtk.Toolbar.__init__(self)
 
+		self.tooltips	= gtk.Tooltips()
 
-	def append_widget(self, widget):
+
+	def append_space(self):
+		"Appends a space to the toolbar"
+
+		space = gtk.SeparatorToolItem()
+		space.set_draw(False)
+
+		self.insert(space, -1)
+
+
+	def append_widget(self, widget, tooltip = None):
 		"Appends a widget to the toolbar"
 
 		toolitem = gtk.ToolItem()
 		toolitem.add(widget)
 
+		if tooltip != None:
+			toolitem.set_tooltip(self.tooltips, tooltip)
+
 		self.insert(toolitem, -1)
 
 
@@ -465,11 +471,11 @@ class PasswordLabel(EventBox):
 	def __init__(self, password = "", cfg = None, clipboard = None, justify = gtk.JUSTIFY_LEFT):
 		EventBox.__init__(self)
 
-		self.password	= password
+		self.password	= util.unescape_markup(password)
 		self.config	= cfg
 		self.clipboard	= clipboard
 
-		self.label = Label(password, justify)
+		self.label = Label(util.escape_markup(self.password), justify)
 		self.label.set_selectable(True)
 		self.add(self.label)
 
@@ -513,7 +519,7 @@ class PasswordLabel(EventBox):
 		"Sets whether to display the password"
 
 		if show == True:
-			self.label.set_text(self.password)
+			self.label.set_text(util.escape_markup(self.password))
 			self.label.set_selectable(True)
 			self.drag_source_unset()
 
@@ -579,7 +585,8 @@ class ComboBoxEntry(gtk.ComboBoxEntry):
 	def __init__(self, list = []):
 		gtk.ComboBoxEntry.__init__(self)
 
-		self.child.set_activates_default(True)
+		self.entry = self.child
+		self.entry.set_activates_default(True)
 
 		self.model = gtk.ListStore(gobject.TYPE_STRING)
 		self.set_model(self.model)
@@ -589,7 +596,7 @@ class ComboBoxEntry(gtk.ComboBoxEntry):
 		self.completion.set_model(self.model)
 		self.completion.set_text_column(0)
 		self.completion.set_minimum_key_length(1)
-		self.child.set_completion(self.completion)
+		self.entry.set_completion(self.completion)
 
 		if list is not None:
 			self.set_values(list)
@@ -598,17 +605,17 @@ class ComboBoxEntry(gtk.ComboBoxEntry):
 	def get_text(self):
 		"Returns the text of the entry"
 
-		return self.child.get_text()
+		return self.entry.get_text()
 
 
 	def set_text(self, text):
 		"Sets the text of the entry"
 
 		if text is None:
-			self.child.set_text("")
+			self.entry.set_text("")
 
 		else:
-			self.child.set_text(text)
+			self.entry.set_text(text)
 
 
 	def set_values(self, list):
@@ -646,8 +653,11 @@ class FileEntry(HBox):
 
 		try:
 			fsel = dialog.FileSelector(None, self.title, self.type)
+			file = self.get_filename()
+
+			if file != None:
+				fsel.set_filename(file)
 
-			fsel.set_filename(self.get_filename())
 			self.set_filename(fsel.run())
 
 		except dialog.CancelError:
@@ -657,7 +667,7 @@ class FileEntry(HBox):
 	def get_filename(self):
 		"Gets the current filename"
 
-		return self.entry.get_text()
+		return io.file_normpath(self.entry.get_text())
 
 
 	def get_text(self):
@@ -684,44 +694,213 @@ gobject.signal_new("changed", FileEntry, gobject.SIGNAL_ACTION, gobject.TYPE_BOO
 
 
 
-class PasswordEntry(Entry):
-	"An entry for editing a password (follows the 'show passwords' preference"
+class IconEntry(Alignment):
+	"An entry with an icon in it"
+
+	def __init__(self, text = None):
+		Alignment.__init__(self)
+
+		self.tooltips	= gtk.Tooltips()
+		self.icon	= None
+		self.icontip	= None
+
+		# set up ui
+		self.hbox = HBox()
+		self.add(self.hbox)
+
+		self.entry = Entry(text)
+		self.entry.set_has_frame(False)
+		self.hbox.pack_start(self.entry)
+
+		self.iconebox	= EventBox()
+		self.iconebox.set_visible_window(False)
+
+		self.iconalign	= Alignment(1.0, 0.5, 0, 0)
+		self.iconalign.set_padding(1, 1, 0, 2)
+		self.iconalign.add(self.iconebox)
+
+		# connect signals
+		self.connect("expose-event", self.__cb_expose)
+		self.connect("size-allocate", self.__cb_size_allocate)
+		self.connect("size-request", self.__cb_size_request)
+
+		self.entry.connect_after("focus-in-event", lambda w,d: self.queue_draw())
+		self.entry.connect_after("focus-out-event", lambda w,d: self.queue_draw())
+
+		self.entry.connect("changed", lambda w: self.emit("changed"))
+		self.entry.connect("populate-popup", lambda w,m: self.emit("populate-popup", m))
+
+
+	def __cb_expose(self, widget, data):
+		"Draws the widget borders on expose"
+
+		allocation	= self.get_allocation()
+		style		= self.entry.get_style()
+		intfocus	= self.entry.style_get_property("interior-focus")
+		focuswidth	= self.entry.style_get_property("focus-line-width")
+
+		x		= allocation.x
+		y		= allocation.y
+		width		= allocation.width
+		height		= allocation.height
+
+		if self.entry.flags() & gtk.HAS_FOCUS == gtk.HAS_FOCUS and intfocus == False:
+			x	+= focuswidth
+			y	+= focuswidth
+			width	-= 2 * focuswidth
+			height	-= 2 * focuswidth
+
+		style.paint_flat_box(self.window, self.entry.state, gtk.SHADOW_NONE, None, self.entry, "entry_bg", x, y, width, height)
+		style.paint_shadow(self.window, gtk.STATE_NORMAL, gtk.SHADOW_IN, None, self.entry, "entry", x, y, width, height)
+
+		if self.entry.flags() & gtk.HAS_FOCUS == gtk.HAS_FOCUS and intfocus == False:
+			x	-= focuswidth
+			y	-= focuswidth
+			width	+= 2 * focuswidth
+			height	+= 2 * focuswidth
+
+			style.paint_focus(self.window, self.entry.state, None, self.entry, "entry", x, y, width, height)
+
+
+	def __cb_size_allocate(self, widget, allocation):
+		"Modifies the widget size allocation"
+
+		child_allocation	= gtk.gdk.Rectangle()
+		xborder, yborder	= self.__entry_get_borders()
+
+		child_allocation.x	= allocation.x + self.border_width + xborder
+		child_allocation.y	= allocation.y + self.border_width + yborder
+		child_allocation.width	= max(allocation.width - (self.border_width + xborder) * 2, 0)
+		child_allocation.height	= max(allocation.height - (self.border_width + yborder) * 2, 0)
+
+		self.hbox.size_allocate(child_allocation)
+		self.queue_draw()
+
+
+	def __cb_size_request(self, widget, requisition):
+		"Modifies the widget size request"
+
+		requisition.width	= self.border_width * 2
+		requisition.height	= self.border_width * 2
+		xborder, yborder	= self.__entry_get_borders()
+
+		entrywidth, entryheight	= self.entry.size_request()
+		requisition.height	+= entryheight >= 18 and entryheight or 18
+
+		requisition.width	+= 2 * xborder
+		requisition.height	+= 2 * yborder
+
+
+	def __entry_get_borders(self):
+		"Returns the border sizes of an entry"
+
+		style		= self.entry.get_style()
+		intfocus	= self.entry.style_get_property("interior-focus")
+		focuswidth	= self.entry.style_get_property("focus-line-width")
+
+		xborder		= style.xthickness
+		yborder		= style.ythickness
+
+		if intfocus == False:
+			xborder	+= focuswidth
+			yborder	+= focuswidth
+
+		return xborder, yborder
+
+
+	def get_text(self):
+		"Wrapper for the entry"
+
+		return self.entry.get_text()
+
+
+	def remove_icon(self):
+		"Removes the icon from the entry"
+
+		self.set_icon(None, "")
+
+
+	def set_icon(self, stock, tooltip = ""):
+		"Sets the icon for the entry"
+
+		if tooltip != self.icontip:
+			self.tooltips.set_tip(self.iconebox, tooltip)
+			self.icontip = tooltip
+
+		if self.icon != None and self.icon.get_stock()[0] == stock:
+			return
+
+		if self.iconalign in self.hbox.get_children():
+			self.hbox.remove(self.iconalign)
+			self.iconebox.remove(self.icon)
+			self.icon.destroy()
+			self.icon = None
+
+		if stock == None:
+			return
+
+		self.icon = Image(stock, ICON_SIZE_ENTRY)
+		self.iconebox.add(self.icon)
+		self.hbox.pack_start(self.iconalign, False, False)
+		self.hbox.show_all()
+
+
+	def set_text(self, text):
+		"Wrapper for the entry"
+
+		self.entry.set_text(text)
+
+
+	def set_visibility(self, visibility):
+		"Wrapper for the entry"
+
+		self.entry.set_visibility(visibility)
+
+
+gobject.type_register(IconEntry)
+gobject.signal_new("changed", IconEntry, gobject.SIGNAL_ACTION, gobject.TYPE_BOOLEAN, ())
+gobject.signal_new("populate-popup", IconEntry, gobject.SIGNAL_ACTION, gobject.TYPE_BOOLEAN, (gobject.TYPE_PYOBJECT, ))
+
+
+
+class PasswordEntry(IconEntry):
+	"An entry for editing a password (follows the 'show passwords' preference)"
 
 	def __init__(self, password = None, cfg = None, clipboard = None):
-		Entry.__init__(self, password)
+		IconEntry.__init__(self, password)
 		self.set_visibility(False)
 
+		self.autocheck	= True
 		self.config	= cfg
 		self.clipboard	= clipboard
 
+		self.connect("changed", self.__cb_check_password)
+		self.connect("populate-popup", self.__cb_popup)
+
 		if cfg != None:
 			self.config.monitor("view/passwords", lambda k,v,d: self.set_visibility(v))
 
-		self.connect("changed", self.__cb_check_password)
-		self.connect("focus-in-event", self.__cb_check_password)
-		self.connect("focus-out-event", self.__cb_check_password)
-		self.connect("populate-popup", self.__cb_popup)
-
 
 	def __cb_check_password(self, widget, data = None):
 		"Callback for changed, checks the password"
 
+		if self.autocheck == False:
+			return
+
 		password = self.get_text()
 
-		if len(password) == 0 or self.is_focus() == False:
-			color = Entry().rc_get_style().base[gtk.STATE_NORMAL]
+		if len(password) == 0:
+			self.remove_icon()
 
 		else:
 			try:
 				util.check_password(password)
 
-			except ValueError:
-				color = gtk.gdk.color_parse("#ffbaba")
+			except ValueError, reason:
+				self.set_password_strong(False, "The password %s" % str(reason))
 
 			else:
-				color = gtk.gdk.color_parse("#baffba")
-
-		self.modify_base(gtk.STATE_NORMAL, color)
+				self.set_password_strong(True, "The password seems good")
 
 
 	def __cb_popup(self, widget, menu):
@@ -736,6 +915,12 @@ class PasswordEntry(Entry):
 		menu.show_all()
 
 
+	def set_password_strong(self, strong, reason = ""):
+		"Sets whether the password is strong or not"
+
+		self.set_icon(strong == True and STOCK_PASSWORD_STRONG or STOCK_PASSWORD_WEAK, reason)
+
+
 
 class PasswordEntryGenerate(HBox):
 	"A password entry with a generator button"
@@ -744,12 +929,14 @@ class PasswordEntryGenerate(HBox):
 		HBox.__init__(self)
 		self.config = cfg
 
-		self.entry = PasswordEntry(password, cfg, clipboard)
-		self.pack_start(self.entry)
+		self.pwentry = PasswordEntry(password, cfg, clipboard)
+		self.pack_start(self.pwentry)
 
 		self.button = Button("Generate", lambda w: self.generate())
 		self.pack_start(self.button, False, False)
 
+		self.entry = self.pwentry.entry
+
 
 	def generate(self):
 		"Generates a password for the entry"
@@ -758,19 +945,19 @@ class PasswordEntryGenerate(HBox):
 		avoid_ambiguous = self.config.get("passwordgen/avoid_ambiguous")
 		password = util.generate_password(length, avoid_ambiguous)
 
-		self.entry.set_text(password)
+		self.pwentry.set_text(password)
 
 
 	def get_text(self):
 		"Wrapper for the entry"
 
-		return self.entry.get_text()
+		return self.pwentry.get_text()
 
 
 	def set_text(self, text):
 		"Wrapper for the entry"
 
-		self.entry.set_text(text)
+		self.pwentry.set_text(text)
 
 
 
@@ -865,6 +1052,12 @@ class DropDown(gtk.ComboBox):
 		return self.model.get(self.model.iter_nth_child(None, index), 0, 1, 2)
 
 
+	def get_num_items(self):
+		"Returns the number of items in the dropdown"
+
+		return self.model.iter_n_children(None)
+
+
 	def insert_item(self, index, text, stock = None, data = None):
 		"Inserts an item in the dropdown"
 
@@ -1065,6 +1258,9 @@ class TreeView(gtk.TreeView):
 	def get_active(self):
 		"Get the currently active row"
 
+		if self.model == None:
+			return None
+
 		iter = self.model.get_iter(self.get_cursor()[0])
 
 		if iter is None or self.selection.iter_is_selected(iter) == False:
@@ -1267,7 +1463,7 @@ class ItemFactory(gtk.IconFactory):
 			icons[STOCK_ENTRY_FOLDER_OPEN] = "revelation-fallback-folder-open"
 
 		for id, name in icons.items():
-			self.load_stock_icon(id, name, ( gtk.ICON_SIZE_MENU, ICON_SIZE_DATAVIEW, ICON_SIZE_DROPDOWN, ICON_SIZE_TREEVIEW ))
+			self.load_stock_icon(id, name, ( ICON_SIZE_DATAVIEW, ICON_SIZE_DROPDOWN, ICON_SIZE_ENTRY, ICON_SIZE_TREEVIEW ))
 
 
 	def __init_items(self):
@@ -1285,17 +1481,18 @@ class ItemFactory(gtk.IconFactory):
 			( STOCK_LOCK,		"_Lock",	"stock_lock" ),
 			( STOCK_NEW_ENTRY,	"_Add Entry",	gtk.STOCK_ADD ),
 			( STOCK_NEW_FOLDER,	"_Add Folder",	"stock_folder" ),
-			( STOCK_NEXT,		"Ne_xt",	gtk.STOCK_GO_FORWARD ),
+			( STOCK_NEXT,		"Next",		gtk.STOCK_GO_DOWN ),
 			( STOCK_OVERWRITE,	"_Overwrite",	gtk.STOCK_SAVE_AS ),
 			( STOCK_PASSWORD_CHANGE,"_Change",	"stock_lock-ok" ),
 			( STOCK_PASSWORD_CHECK,	"_Check",	"stock_lock-ok" ),
 			( STOCK_PASSWORD_STRONG,"",		"stock_lock-ok" ),
 			( STOCK_PASSWORD_WEAK,	"",		"stock_lock-broken" ),
-			( STOCK_PREVIOUS,	"Pre_vious",	gtk.STOCK_GO_BACK ),
+			( STOCK_PREVIOUS,	"Previous",	gtk.STOCK_GO_UP ),
 			( STOCK_RELOAD,		"_Reload",	gtk.STOCK_REFRESH ),
 			( STOCK_REMOVE,		"Re_move",	gtk.STOCK_DELETE ),
 			( STOCK_UNKNOWN,	"Unknown",	gtk.STOCK_DIALOG_QUESTION ),
 			( STOCK_UNLOCK,		"_Unlock",	"stock_lock-open" ),
+			( STOCK_UPDATE,		"_Update",	"stock_edit" ),
 		)
 
 		for id, name, icon in items:
@@ -1343,7 +1540,7 @@ class ItemFactory(gtk.IconFactory):
 		iconset = gtk.IconSet()
 
 		if self.theme.has_icon(icon):
-			for size in sizes:
+			for size in dict.fromkeys(sizes).keys():
 				pixelsize = gtk.icon_size_lookup(size)[0]
 
 				source = gtk.IconSource()
@@ -1725,18 +1922,36 @@ class Searchbar(Toolbar):
 	def __init__(self):
 		Toolbar.__init__(self)
 
-		self.label	= Label("  Search for: ")
-		self.entry	= Entry()
-		self.button	= Button(" Find ")
+		self.label		= Label("  Find: ")
+		self.entry		= Entry()
+		self.dropdown		= EntryDropDown()
+		self.dropdown.insert_item(0, "Any type", "gnome-stock-about")
+		self.button_next	= Button(STOCK_NEXT)
+		self.button_prev	= Button(STOCK_PREVIOUS)
 
 		self.append_widget(self.label)
-		self.append_widget(self.entry)
-		self.append_widget(self.button)
+		self.append_widget(self.entry, "Text to search for")
+		self.append_widget(EventBox(self.dropdown), "The type of account to search for")
+		self.append_space()
+		self.append_widget(self.button_next, "Find the next match")
+		self.append_widget(self.button_prev, "Find the previous match")
 
-		self.entry.connect("changed", lambda w: self.button.set_sensitive(self.entry.get_text() != ""))
+		self.connect("show", self.__cb_show)
+
+		self.entry.connect("changed", self.__cb_entry_changed)
 		self.entry.connect("key-press-event", self.__cb_key_press)
 
-		self.button.set_sensitive(False)
+		self.button_next.set_sensitive(False)
+		self.button_prev.set_sensitive(False)
+
+
+	def __cb_entry_changed(self, widget, data = None):
+		"Callback for entry changes"
+
+		s = self.entry.get_text() != ""
+
+		self.button_next.set_sensitive(s)
+		self.button_prev.set_sensitive(s)
 
 
 	def __cb_key_press(self, widget, data = None):
@@ -1744,6 +1959,18 @@ class Searchbar(Toolbar):
 
 		# return
 		if data.keyval == 65293 and widget.get_text() != "":
-			self.button.activate()
+			if data.state & gtk.gdk.SHIFT_MASK == gtk.gdk.SHIFT_MASK:
+				self.button_prev.activate()
+
+			else:
+				self.button_next.activate()
+
 			return True
 
+
+	def __cb_show(self, widget, data = None):
+		"Callback for widget display"
+
+		self.entry.select_region(0, -1)
+		self.entry.grab_focus()
+
