@@ -1,12 +1,12 @@
 #
-# Revelation 0.4.5 - a password manager for GNOME 2
+# Revelation 0.4.7 - a password manager for GNOME 2
 # http://oss.codepoet.no/revelation/
-# $Id: util.py 415 2005-08-17 15:59:33Z erikg $
+# $Id: util.py 476 2006-02-02 19:28:56Z erikg $
 #
 # Module with various utility functions
 #
 #
-# Copyright (c) 2003-2005 Erik Grinaker
+# Copyright (c) 2003-2006 Erik Grinaker
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -25,7 +25,7 @@
 
 import crack
 
-import datetime, os, random, shlex, string, StringIO, traceback
+import datetime, math, os, random, shlex, string, StringIO, traceback
 
 
 class SubstFormatError(Exception):
@@ -39,73 +39,80 @@ class SubstValueError(Exception):
 
 
 
-def check_password(password, check_len = True, check_var = True, check_str = True, check_crack = True):
+def check_password(password):
 	"Checks if a password is valid"
 
 	# check for length
-	if check_len == True:
-		if len(password) < 6:
-			raise ValueError, "is too short"
+	if len(password) < 6:
+		raise ValueError, "is too short"
 
 
-	# check for different characters
-	if check_var == True:
+	# check for entropy
+	pwlen	= len(password)
+	ent	= entropy(password)
+	idealent = entropy_ideal(pwlen)
 
-		chars = {}
-		for char in password:
-			if not chars.has_key(char):
-				chars[char] = 0
-
-			chars[char] += 1
-
-		if len(chars) < 6:
-			raise ValueError, "isn't varied enough"
-
-
-		# check if the password is a palindrome
-		for i in range(len(password)):
-			if password[i] != password[-i - 1]:
-				break
-
-		else:
-			raise ValueError, "is a palindrome"
+	if (pwlen < 100 and ent / idealent < 0.8) or (pwlen >= 100 and ent < 5.3):
+		raise ValueError, "isn't varied enough"
 
 
 	# check the password strength
-	if check_str == True:
-		limit		= 10
-		cred_lower	= 1.1
-		cred_upper	= 1.4
-		cred_digit	= 2.0
-		cred_other	= 3.0
+	lc, uc, d, o = 0, 0, 0, 0
+
+	for c in password:
+		if c in string.ascii_lowercase:
+			lc += 1
+
+		elif c in string.ascii_uppercase:
+			uc += 1
+
+		elif c in string.digits:
+			d += 1
+
+		else:
+			o += 1
+
+	classcount = [ lc, uc, d, o ]
+	classcount.sort()
+	classcount.reverse()
+
+	cred = sum([ count * (1 + (weight ** 2.161 / 10)) for weight, count in zip(range(1, len(classcount) + 1), classcount) ])
+
+	if cred < 10:
+		raise ValueError, "is too weak"
 
 
-		cred = 0
+	# check if the password is a palindrome
+	for i in range(len(password)):
+		if password[i] != password[-i - 1]:
+			break
 
-		for c in password:
-			if c in string.ascii_lowercase:
-				cred += cred_lower
-
-			elif c in string.ascii_uppercase:
-				cred += cred_upper
-
-			elif c in string.digits:
-				cred += cred_digit
-
-			else:
-				cred += cred_other
-
-		if cred < limit:
-			raise ValueError, "is too weak"
+	else:
+		raise ValueError, "is a palindrome"
 
 
 	# check password with cracklib
-	if check_crack == True:
-		try:
+	try:
+		if len(password) < 100:
 			crack.FascistCheck(password)
 
-		except IOError:
-			pass
+	except ValueError, reason:
+
+		# modify reason
+		reason = str(reason).strip()
+		reason = reason.replace("simplistic/systematic", "systematic")
+		reason = reason.replace(" dictionary", "")
+
+		if reason[:3] == "it ":
+			reason = reason[3:]
+
+		if reason[:5] == "it's ":
+			reason = "is " + reason[5:]
+
+		raise ValueError, reason
+
+	except IOError:
+		pass
 
 
 def dom_text(node):
@@ -120,6 +127,26 @@ def dom_text(node):
 	return text
 
 
+def entropy(string):
+	"Calculates the Shannon entropy of a string"
+
+	# get probability of chars in string
+	prob = [ float(string.count(c)) / len(string) for c in dict.fromkeys(list(string)) ]
+
+	# calculate the entropy
+	entropy = - sum([ p * math.log(p) / math.log(2.0) for p in prob ])
+
+	return entropy
+
+
+def entropy_ideal(length):
+	"Calculates the ideal Shannon entropy of a string with given length"
+
+	prob = 1.0 / length
+
+	return -1.0 * length * prob * math.log(prob) / math.log(2.0)
+
+
 def escape_markup(string):
 	"Escapes a string so it can be placed in a markup string"
 
@@ -149,52 +176,61 @@ def execute_child(command):
 	"Runs a command as a child, returns its process ID"
 
 	items = shlex.split(command.encode("iso-8859-1"), 0)
+
 	return os.spawnvp(os.P_NOWAIT, items[0], items)
 
 
 def generate_password(length, avoidambiguous = False):
 	"Generates a password"
 
-	password = []
+	# set up character sets
+	d	= string.digits
+	lc	= string.ascii_lowercase
+	uc	= string.ascii_uppercase
 
-	while len(password) < length:
+	if avoidambiguous == True:
+		d	= d.translate(string.maketrans("", ""), "015")
+		lc	= lc.translate(string.maketrans("", ""), "lqg")
+		uc	= uc.translate(string.maketrans("", ""), "IOS")
 
-		if len(password) < int(round(length * 0.15)):
-			set = string.digits
+	fullset = d + uc + lc
 
-		elif len(password) < int(round(length * (0.15 + 0.24))):
-			set = string.ascii_lowercase
+	charsets = (
+		( d,	0.15 ),
+		( uc,	0.24 ),
+		( lc,	0.24 ),
+	)
 
-		elif len(password) < int(round(length * (0.15 + 0.24 + 0.24))):
-			set = string.ascii_uppercase
+	
+	# function for generating password
+	def genpw(length):
+		password = []
 
-		else:
-			set = string.ascii_letters + string.digits
+		for set, share in charsets:
+			password.extend([ random.choice(set) for i in range(int(round(length * share))) ])
 
+		while len(password) < length:
+			password.append(random.choice(fullset))
 
-		char = set[int(random.random() * len(set))]
+		random.shuffle(password)
 
-		if avoidambiguous == True and char in "0OIl1S5qg":
-			continue
+		return "".join(password)
 
-		password.append(char)
-
-
-	random.shuffle(password)
-
-	password = "".join(password)
 
 	# check password, and regenerate if needed
-	if length < 8:
-		return password
-
 	while 1:
 		try:
+			password = genpw(length)
+
+			if length <= 6:
+				return password
+
 			check_password(password)
+
 			return password
 
 		except ValueError:
-			password = generate_password(length)
+			continue
 
 
 def pad_right(string, length, padchar = " "):
@@ -335,3 +371,13 @@ def trace_exception(type, value, tb):
 
 	return trace.getvalue()
 
+
+def unescape_markup(string):
+	"Unescapes a string to get literal values"
+
+	string = string.replace("&amp;", "&")
+	string = string.replace("&lt;", "<")
+	string = string.replace("&gt;", ">")
+
+	return string
+
